export default {


    data() {
        return {
            loading: true,
        }
    },

    props: {
        forms: {
            type: Array,
            default: () => []
        }
    },

    mounted() {
        this.forms[0]._visible = true;
        this.load();
    },

    methods: {
        async load() {

            let promises = [];

            this.loading = true;

            this.forms.forEach(form => {

                const p = new Promise((resolve, reject) => {

                    let options = {
                        sort: {_created: -1},
                        limit: 5,
                    };

                    this.$request(`/inbox/submissions/load/${form._id}`, {options}).then(rsp => {
                        form.submissions = rsp.items;
                        resolve();
                    });
                });

                promises.push(p);
            });

            Promise.all(promises).then(() => {
                this.loading = false;
            });
        },

        preview(submission) {
            return JSON.stringify(Object.values(submission.data || {})).replace(/('null'|\[|\]|\{|\}|"|\\|')/g, '').replace(/,/g, ', ').substr(0, 50).trim();
        },

        showSubmission(submission) {
            VueView.ui.offcanvas('inbox:assets/dialogs/submission.js', {submission}, {}, {flip: true, size: 'xlarge'})
        }
    },

    template: /*html*/`
        <kiss-card>
            <div class="kiss-text-caption kiss-text-bold">{{ t('Inbox') }}</div>
            <div class="kiss-color-muted kiss-size-small kiss-margin-xsmall">{{ t('Latest form submissions') }}</div>

            <div class="kiss-padding-large" v-if="loading"><app-loader size="small"></app-loader></div>

            <div class="animated fadeIn kiss-margin-small" v-if="!loading">

                <kiss-card class="kiss-padding-small kiss-margin-small" theme="bordered" hover="contrast shadowed bordered-primary" v-for="form in forms">

                    <div class="kiss-flex kiss-flex-middle kiss-margin kiss-visible-toggle" gap="small">
                        <div>
                            <kiss-svg :src="$baseUrl(form.icon ? form.icon : 'inbox:icon.svg')" width="25" height="25" :style="{color:form.color ? form.color : 'var(--kiss-color-muted)'}"><canvas width="25" height="25"></canvas></kiss-svg>
                        </div>
                        <a class="kiss-flex-1 kiss-text-bold kiss-text-capitalize kiss-link-muted" :href="$routeUrl('/inbox/submissions/form/'+form._id)">{{ form.name}}</a>
                        <a class="kiss-link-muted kiss-flex kiss-flex-middle" gap="small" @click="form._visible = !form._visible">
                            <span class="kiss-badge">{{form.submissions.length}}</span>
                            <icon>unfold_more</icon>
                        </a>
                    </div>

                    <div class="animated fadeIn kiss-margin-small" v-if="form._visible">

                        <a class="kiss-display-block kiss-link-muted kiss-margin-small kiss-size-small" gap="small" @click="showSubmission(submission)" v-for="submission in form.submissions">
                            <div>{{ preview(submission) }}</div>
                            <div class="kiss-text-monospace kiss-size-xsmall kiss-color-muted"><app-datetime type="relative" :datetime="submission._created" /></div>
                        </a>
                    </div>

                </kiss-card>

            </div>

            <div class="kiss-margin-small" v-if="!loading">
                <a class="kiss-button kiss-button-small" :href="$routeUrl('/inbox')">{{ t('Go to Inbox') }}</a>
            </div>

        </kiss-card>
    `
}
